@echo off
setlocal enabledelayedexpansion
set optBuilt=0

for /d %%a in (%*) do (
    if %%a == 32 set optBuilt=1&call:BuildCurl 32
    if %%a == 64 set optBuilt=1&call:BuildCurl 64
)

if %optBuilt% == 0 (
    echo.
    echo Usage: %~nx0 ^<32 ^| 64 ^| 32 64^> 
    echo Specify what cpu to use - 32 or 64 bits or even both.
    echo.
    exit /b
)

exit /b

:BuildCurl
set cpu=%1
if "%LogDir%" == "" set LogDir=%~dp0

call:Download http://www.cmake.org/files/v2.8/cmake-2.8.11-win32-x86.zip cmake-2.8.11-win32-x86.zip
if errorlevel 1 exit /b 1

call:Extract  cmake-2.8.11-win32-x86.zip cmake-2.8.11-win32-x86
if errorlevel 1 exit /b 1

rem set OpenSslPackage=openssl-1.0.1d.tar.gz
rem ^ Non-working package
set OpenSslPackage=openssl-0.9.8y.tar.gz
call:Download http://www.openssl.org/source/%OpenSslPackage% %OpenSslPackage%
if errorlevel 1 exit /b 1

call:GetFilenameOnly OpenSslPackage1 %OpenSslPackage%
call:Extract  %OpenSslPackage% %OpenSslPackage1%
if errorlevel 1 exit /b 1

call:GetFilenameOnly OpenSslPackage2 %OpenSslPackage1%
call:Extract  %OpenSslPackage1% %cpu%bit\%OpenSslPackage2% -o%cpu%bit
if errorlevel 1 exit /b 1

call:Download https://codeload.github.com/bagder/curl/zip/master curl-master.zip
if errorlevel 1 exit /b 1

call:Extract  curl-master.zip curl-master
if errorlevel 1 exit /b 1

rem --------------------------------------------------------------------------------------
rem OpenSSL libraries
rem --------------------------------------------------------------------------------------
if exist %cpu%bit\%OpenSslPackage2%\out32dll\libeay32.lib goto NoNeedToBuildOpenSsh
echo Building openssl ^(%cpu%-bit^)...
pushd %cpu%bit\%OpenSslPackage2%
setlocal enabledelayedexpansion

if %cpu% == 64 (
    call "%VS100COMNTOOLS%\..\..\vc\vcvarsall.bat" amd64 >nul
    call perl Configure VC-WIN64A >logBuild.txt 2>&1
    rem md tmp32dll 2>nul
    rem call perl crypto\x86_64cpuid.pl tmp32dll\x86_64cpuid.asm
    call ms\do_win64a >>logBuild.txt 2>&1
    call nmake -f ms\ntdll.mak >>logBuild.txt 2>&1
)

if %cpu% == 32 (
    call "%VS100COMNTOOLS%\..\..\vc\vcvarsall.bat" x86 >nul
    call perl Configure VC-WIN32 >logBuild.txt 2>&1
    call ms\do_ms >>logBuild.txt 2>&1
    call nmake -f ms\ntdll.mak >>logBuild.txt 2>&1
)

endlocal
popd

:NoNeedToBuildOpenSsh

md curl-master\build\%cpu%bit >nul 2>&1
pushd curl-master\build\%cpu%bit

rem --------------------------------------------------------------------------------------
rem Generate solution file
rem --------------------------------------------------------------------------------------
if exist CURL.sln goto ProjectMade

set GConf="Visual Studio 10 Win64"
if %cpu% == 32 set GConf="Visual Studio 10"

echo Generating %cpu%-bit project files ^(log in logCmake.txt^)...
..\..\..\cmake-2.8.11-win32-x86\bin\cmake.exe -DLIB_EAY_RELEASE=..\..\..\%cpu%bit\%OpenSslPackage2%\out32dll\libeay32.lib -DSSL_EAY_RELEASE=..\..\..\%cpu%bit\%OpenSslPackage2%\out32dll\ssleay32.lib -DOPENSSL_ROOT_DIR=..\..\..\%cpu%bit\%OpenSslPackage2% -DOPENSSL_LIBRARIES=..\..\..\%cpu%bit\%OpenSslPackage2%\out32dll -DOPENSSL_INCLUDE_DIR=..\..\..\%cpu%bit\%OpenSslPackage2%\inc32 -DBUILD_CURL_TESTS=0 -G %GConf% ..\.. >%LogDir%logCmake.txt 2>&1
if errorlevel 1 (
    echo Error: Failed to make CURL solution file - See log in logCmake.txt.
    popd
    exit /b 1
)
del /f %LogDir%logCmake.txt >nul

:ProjectMade


rem --------------------------------------------------------------------------------------
rem Build solution file
rem --------------------------------------------------------------------------------------
rem set BuildConf=Release
rem set BuildConf=ReleaseWithDebInfo
rem ^ Alternative configurations...
rem
set BuildConf=MinSizeRel

if exist lib\%BuildConf%\libcurl.dll goto FullyDone
echo Building %cpu%-bit curl binaries ^(log in logBuild.txt^)...
setlocal enabledelayedexpansion
if %cpu% == 64 call "%VS100COMNTOOLS%\..\..\vc\vcvarsall.bat" amd64 >nul
if %cpu% == 32 call "%VS100COMNTOOLS%\..\..\vc\vcvarsall.bat" x86 >nul

set VsCpu=x64
if %cpu% == 32 set VsCpu=Win32
echo ^>devenv /nologo /build "%BuildConf%|%VsCpu%" CURL.sln >%LogDir%logBuild.txt
devenv /nologo /build "%BuildConf%|%VsCpu%" CURL.sln >>%LogDir%logBuild.txt 2>&1
endlocal

if errorlevel 1 (
    echo Error: Failed to build CURL binaries - See log in logBuild.txt.
    popd
    exit /b 1
)
del /f %LogDir%logBuild.txt >nul

:FullyDone

echo Copying built binaries to release\%cpu%bit...

call:CopyToRelease lib\%BuildConf%\libcurl.dll
if errorlevel 1 popd & endlocal & exit /b 1

call:CopyToRelease src\%BuildConf%\curl.exe
if errorlevel 1 popd & endlocal & exit /b 1
popd

call:CopyToRelease %cpu%bit\%OpenSslPackage2%\out32dll\libeay32.dll
if errorlevel 1 popd & endlocal & exit /b 1

call:CopyToRelease %cpu%bit\%OpenSslPackage2%\out32dll\ssleay32.dll
if errorlevel 1 popd & endlocal & exit /b 1

echo.
echo Built binaries are available in: 
echo    release\%cpu%bit\
echo.

endlocal
exit /b

:CopyToRelease
    echo f | xcopy /i /q /y %1 %~dp0build\%cpu%bit\ >log.txt 2>&1
    if errorlevel 1 (
        echo Error: Failed to copy %1 to release\64bit\ - 
        type log.txt
        exit /b 1
    )
    del /f log.txt >nul
    exit /b 0


rem ---------------------------------------------------------
rem Get %1 URL to %2 file.
rem ---------------------------------------------------------
:Download
    if exist %2 exit /b 0
    echo Downloading %1...
    curl %1 -o %2 -k >log.txt 2>&1
    if errorlevel 1 (
        echo.
        echo Error: Failed to download %1 -
        type log.txt
        del /f log.txt >nul
        exit /b 1 
    )
    del /f log.txt >nul
    exit /b 0

rem ---------------------------------------------------------
rem Extracts %1 zip to %2 folder
rem ---------------------------------------------------------
:Extract
    if exist %2 exit /b 0

    echo Extracting %1 to %2...
    7za x %1 %3 %4 %5 %6 %7 %8 %9>log.txt
    if errorlevel 1 (
        echo Error: Failed to extract %1 file ^(Log in log.txt^)
        exit /b 1
    )
    del /f log.txt >nul
    exit /b 0


:GetFilenameOnly
    set "%~1=%~n2"
    exit /b
